<?php

namespace Controller;

use Form\LoginForm;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Breadcrumb;
use Texte;
use Client;
use Pays;
use Panier;
use Boutique;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class LoginController
 * @package Controller
 *
 */
class LoginController extends Controller
{

    /**
     * @Route("/login.html", name="login", methods = { "GET" })
     *
     * @return mixed
     */
    public function show_login()
    {
        if ($this->getCurrentClient()) {
            return $this->redirect($this->generateUrl('compte'));
        }

        $texte = new Texte();
        Breadcrumb::getInstance()->add($texte->getOneOrCreate('connexion', 'Connexion'));

        $pays = new Pays();
        $client = new Client();

        $tplVars = array();
        $tplVars['pays'] = $pays->get();
        $tplVars['civilite'] = $client->getCivilite();

        $origin = $this->request->get('origin',null);

        if (!empty($origin)) {
            $tplVars["post"]["origin"] = $origin;
            $tplVars["get"]["origin"] = $origin;
        }

        $tplVars['in_cart'] = (!empty($origin) && 'panier' === $origin);

        if($tplVars['in_cart']){
            list($totalHt, $totalTtc) = Panier::getInstance()->getTotalProductsAndShip();
            $tplVars['montant'] = array(
                'ht' => $totalHt,
                'ttc' => $totalTtc
            );
        }

        return $this->render('login.tpl', $tplVars);
    }

    /**
     * @Route("/login.html", name="login_post", methods = { "POST" })
     */
    public function post_login()
    {
        $client = new Client();
        $texte = new Texte();
        $post = $this->request->request->all();
        $session = $this->request->getSession();
        $flashBag = $this->getFlashBag();
        $form = new LoginForm();

        if ($error = $form->handle($this->request)) {
            $json['field_errors'] = json_encode($error);
        }

        if (!empty($post['origin'])) {
            $origin = $post['origin'];
        } else {
            $origin = $this->request->get('origin');
        }

        if ($error = $form->handle($this->request)) {
            $flashBag->add("error", $error);
        }

        if(empty($error)) {
            $arrClient = $client->get(array('client_email' => $post['client_email']));
            if (!empty($arrClient[0]['client_id']) &&  \Tools::passwordValidate($post['client_password'], $arrClient[0]['client_password'])) {
                $client->setSession($session, $arrClient[0], array("civilite_nom_nom"));
                $client->connexion(array('client_id' => $arrClient[0]['client_id']));

                if (!empty($origin) && $origin == "panier") {
                    Panier::getInstance()->setCustomer(new Client($session->get('client_id')));
                    $json['redirect'] = $this->generateUrl(Boutique::getInstance()->getNextPageProcess());
                } else {
                    $json['redirect'] = $this->generateUrl('compte');
                }
            } else {
                $json['error'] = $texte->getOneOrCreate("errLoginCred", "Informations de connexion invalides.");
            }
        }

        return $this->response($json);
    }

    /**
     * @Route("/logout.html", name="logout", defaults = {"role" = "customer"})
     *
     * @return RedirectResponse
     */
    public function logout()
    {
        $client = new Client();
        $client->destroySession($this->request->getSession());

        return $this->redirect($this->generateUrl('home'));
    }
}
