<?php

/**
 * Class Imager
 */
class Imager
{
    private $_imagesPath;
    private $_imagesWsPath;
    private $_cachePath;
    private $_cacheWsPath;
    private $_watermarkDisplay;
    private $_watermarkImage;
    private $_watermarkPosition;
    private $_watermarkOpacity;

    public function __construct()
    {
        $this->_imagesPath = MEDIA_DIR;
        $this->_imagesWsPath = '/media';
        $this->_cachePath = SITE_DIR . 'web/media/cache';
        $this->_cacheWsPath = '/media/cache';
        $this->_watermarkDisplay = false; // need to be set via config options
        $this->_watermarkImage = ROOT . '/media/watermark.png'; // need to be set via config options
        $this->_watermarkPosition = 'bottom_center'; // need to be set via config options
        $this->_watermarkOpacity = 50; // need to be set via config options
    }

    private function _getImgTag($src, $config = array())
    {
        unset($config['seo'], $config['disableWatermark']);
        if (empty($this->view->resourceUrl) || strpos($src, $this->view->resourceUrl) === 0) {
            $config['src'] = $src;
        } else {
            $config['src'] = $this->view->resourceUrl . $src;
        }
        $config['src'] = $src;

        $width = $config['width'];
        unset($config['width']);

        $height = $config['height'];
        unset($config['height']);

        $html = '<img ';
        foreach ($config as $name => $value) {
            $html .= $name . '="' . $value . '" ';
        }
        $html .= 'width="'.$width.'" height="'.$height.'" />';
        return $html;
    }

    /**
     * Image resizer, cacher, seo, watermark
     * Possible config keys:
     * width => image width
     * height => image height
     * alt => alternative text & title attribute
     * seo => seo image name
     * disableWatermark => false
     *
     * @param string $src
     * @param array $config
     * @return string image tag
     */
    public function imager($src, $config = array(), $tag = true)
    {

        $config = array_merge(array(
            'width' => 0,
            'height' => 0,
            'seo' => '',
            'alt' => '',
            'disableWatermark' => true,
            'quality' => null,
        ), array_filter($config));

        $src = trim($src, '/\\');

        if (empty($src) || !is_file($this->_imagesPath . $src)) {
            $src = $this->_imagesPath . 'no_image.png';
            $filename = 'no_image';

        } else {

            $src = $this->_imagesPath . $src;

            $imagename = substr($src, strrpos($src, DS) + 1);
            $imagename = substr($imagename, 0, strrpos($imagename, '.'));
            if (empty($config['seo'])) {
                $config['seo'] = $imagename;
            } else {
                $config['seo'] .= '_' . $imagename;
            }

            $filename = $config['seo'];
        }



        // remove all cyrillic symbols to support $filename[$i]
        $filename = strtolower(preg_replace('/[^a-zA-Z0-9_\.]/', '_', $filename));

        if ($config['width'] > 0) {
            $config['height'] = $config['height'] > 0 ? $config['height'] : $config['width'];
            $filename .= '_' . $config['width'] . 'x' . $config['height'];
        }

        $filename .= substr($src, strrpos($src, '.')); // add extension

        $dirDest = str_replace(MEDIA_DIR, '', $src);
        $dirDest = str_replace(basename($src), '', $dirDest);

        $wsDesination = $this->_cacheWsPath . DS . $dirDest . $filename[0] . DS . $filename[1];
        $destination = $this->_cachePath . DS . $dirDest . $filename[0] . DS . $filename[1];

        $filePath = $destination . DS . $filename;
        if (file_exists($filePath) && filemtime($filePath) > filemtime($src)) {
            if ($tag) {
                return $this->_getImgTag($wsDesination . DS . $filename, $config);
            } else {
                return $wsDesination . DS . $filename;
            }
        }

        try {
            $image = new Image($src, $config['quality']);
            $image->resize($config['width'], $config['height']);
            if ($this->_watermarkDisplay && !$config['disableWatermark']) {
                $image->applyWatermark(
                    $this->view->skinPath('images/' . $this->_watermarkImage),
                    $this->_watermarkPosition,
                    $this->_watermarkOpacity
                );
            }
            $image->save($destination, $filename);
        } catch (Exception $e) {
            $src = $this->_imagesPath . 'no_image.png';
            $filename = 'no_image' . '_' . $config['width'] . 'x' . $config['height'] . '.jpg';

            $image = new Image($src, $config['quality']);
            if ($config['width'] || $config['height']) {
                $image->resize($config['width'], $config['height']);
            }
            $image->save($destination, $filename);
        }

        if ($tag) {
            return $this->_getImgTag($wsDesination . DS . $filename, $config);
        } else {
            return $wsDesination . DS . $filename;
        }
    }
}
