<?php

/**
 * Class Livraison
 * @property integer $livraison_id
 * @property float $port_ht
 * @property float $port_ttc
 * @property float $taxe_taux
 * @property string $livraison_nom_nom
 * @property string $livraison_code
 * @property string $delai_date_min
 * @property string $delai_date_max
 * @property integer $livraison_type_id
 */
class Livraison extends Model {

    const LIVRAISON_PAYS_ISO = 'FR';

    const TYPE_DOM = 2;
    const TYPE_REL = 1;
    const LIVRAISON_EXPRESS = 3;

    protected $table = 'livraison';

    /**
     * @param array $formvars
     * @param string $champs
     * @return null
     * recuperation des frais de port
     */
    function get($formvars = array(), $champs = "") {


        $sql = "SELECT *, ln.livraison_nom_desc as livraison_desc, ln.livraison_nom_nom as livraison_nom, l.livraison_id, t.taxe_taux, l.livraison_type_id";

        if (isset($formvars['prix_ttc'])) {
            $formvars['prix_ttc'] = str_replace(',', '.',$formvars['prix_ttc']);
            $sql.=", IF(ROUND(livraison_gratuit_prix * (1 + t2.taxe_taux),2)<=" . $formvars['prix_ttc'] . ", 0, livraison_zone_prix) AS port_ht";
            $sql.=", IF(ROUND(livraison_gratuit_prix * (1 + t2.taxe_taux),2)<=" . $formvars['prix_ttc'] . ", 0, ROUND(livraison_zone_prix * (1 + t.taxe_taux),2)) AS port_ttc";
            $sql.=", (ROUND(livraison_gratuit_prix * (1 + t2.taxe_taux),2)-" . $formvars['prix_ttc'] . ") AS livraison_gratuit_reste";
        }
        if (isset($formvars['prix_ht'])) {
            $formvars['prix_ht'] = str_replace(',', '.',$formvars['prix_ht']);
            $sql.=", IF(livraison_gratuit_prix<=" . $formvars['prix_ht'] . ", 0, livraison_zone_prix) AS port_ht";
            $sql.=", IF(livraison_gratuit_prix<=" . $formvars['prix_ht'] . ", 0, ROUND(livraison_zone_prix * (1 + t.taxe_taux),2)) AS port_ttc";
            $sql.=", (livraison_gratuit_prix - " . $formvars['prix_ht'] . ") AS livraison_gratuit_reste";
        }

        $sql.=" FROM (livraison l)";

        $sql.=" INNER JOIN livraison_zone lz ON(l.livraison_id = lz.livraison_id)";
        $sql.=" INNER JOIN pays p ON(lz.zone_id = p.zone_id)";
        $sql.=" INNER JOIN livraison_nom ln ON(l.livraison_id = ln.livraison_id AND ln.langue_id = '" . Boutique::getInstance()->getLangue()->id . "')";
        $sql.=" INNER JOIN taxe t ON(lz.taxe_id = t.taxe_id)";
        $sql.=" INNER JOIN livraison_boutique lb ON(l.livraison_id = lb.livraison_id AND lb.boutique_id = '".Boutique::getInstance()->id."')";

        if (isset($formvars['prix_ht']) || isset($formvars['prix_ttc'])) {
            $sql.=" LEFT JOIN livraison_gratuit lg ON (lz.zone_id = lg.zone_id AND lz.livraison_id = lg.livraison_id)";
            $sql.=" LEFT JOIN taxe t2 ON (t2.taxe_id = lg.taxe_id)";
            $sql.=" LEFT JOIN zone z ON (z.zone_id = lz.zone_id)";
        }

        $sql.=" LEFT JOIN livraison_type_nom ltn ON(l.livraison_type_id = ltn.livraison_type_id AND ltn.langue_id = '". Boutique::getInstance()->getLangue()->id ."')";

        $sql.=" WHERE 1";
        $sql.=" AND l.livraison_statut_id = 1";
        $sql.=" AND lb.livraison_boutique_statut_id = 1";
        // TODO : revoir la remonté des zones. Remonté uniquement boutique_id IS NULL si pas de ligne pour boutique_id
        $sql.=" AND (lz.boutique_id IS NULL OR lz.boutique_id = '".Boutique::getInstance()->id."')";

        if (!empty($formvars['livraison_id']))
            $sql.=" AND l.livraison_id = '" . $formvars['livraison_id'] . "'";
        if (!empty($formvars['livraison_code']))
            $sql.=" AND l.livraison_code = '" . $formvars['livraison_code'] . "'";
        if (!empty($formvars['zone_id']))
            $sql.=" AND lz.zone_id = '" . $formvars['zone_id'] . "'";
        if (!empty($formvars['livraison_zone_id']))
            $sql.=" AND livraison_zone_id = '" . $formvars['livraison_zone_id'] . "'";
        if (!empty($formvars['pays_id']))
            $sql.=" AND p.pays_id = '" . $formvars['pays_id'] . "'";
        if (isset($formvars['prix']))
            $sql.=" AND livraison_zone_prix_min  <= '" . $formvars['prix'] . "' AND livraison_zone_prix_max  >= '" . $formvars['prix'] . "'";
        if (isset($formvars['prix_ht']))
            $sql.=" AND livraison_prix_min  <= '" . $formvars['prix_ht'] . "' AND livraison_prix_max  >= '" . $formvars['prix_ht'] . "'";
        if (isset($formvars['prix_ttc']))
            $sql.=" AND (livraison_prix_min * (1 + t.taxe_taux))  <= '" . $formvars['prix_ttc'] . "' AND (livraison_prix_max * (1 + t.taxe_taux))  >= '" . $formvars['prix_ttc'] . "'";

        if (isset($formvars['prix_ht']) || isset($formvars['prix_ttc'])) {
            // TODO : revoir pour remonter une valeur ?
            $sql.=" AND (lg.boutique_id IS NULL OR lg.boutique_id = '".Boutique::getInstance()->id."')";
        }

        $sql.=" GROUP BY livraison_zone_id";
        $sql.=" ORDER BY livraison_rang";

        if (empty($champs)) {
            return $this->sql->getAll($sql);
        } else {
            $result = $this->sql->getOne($sql);
            return $result[$champs];
        }
    }

    static public function isRelay(Livraison $livraison)
    {
        return $livraison->livraison_type_id == Livraison::TYPE_REL;
    }

    /**
     * @param $className
     *
     * @return Shopbydedi\Shipping\RelayInterface
     * @throws DediException
     */

    public function getObjectRelay($className)
    {
        $object = new $className(Boutique::getInstance()->getConfig());

        $interface = class_implements($object);
        if (in_array('Shopbydedi\\Shipping\\RelayInterface', $interface) == false) {
            throw new DediException('the class ' . $className . ' doesn\'t implements Shopbydedi\\Shipping\\RelayInterface');
        }

        return $object;
    }

    public function getHTMLRelay($className, $param)
    {
        try {
            $oRelais = $this->getObjectRelay($className);
        } catch (DediException $e) {
            return false;
        }

        $relais = $oRelais->getList($param);

        $html = '';
        if (!empty($relais)) {
            $smarty = new SiteSmarty();
            $smarty->assign('relais', $relais);
            $smarty->assign('param', $param);
            $html = $smarty->fetch('order/relais.tpl');
        }

        return $html;
    }
}