<?php

namespace Shopbydedi\Payment;

use Panier;
use PayPal\Exception\PayPalConnectionException;
use Tpe;
use Boutique;
use Application;
use DediException;
use Symfony\Component\HttpFoundation\RedirectResponse;

use PayPal\Api\Amount;
use PayPal\Api\Details;
use PayPal\Api\Item;
use PayPal\Api\ItemList;
use PayPal\Api\Payer;
use PayPal\Api\Payment;
use PayPal\Api\RedirectUrls;
use PayPal\Api\Transaction;
use PayPal\Api\PaymentExecution;
use PayPal\Rest\ApiContext;
use PayPal\Auth\OAuthTokenCredential;

/**
 * Class Paypal
 * @package Shopbydedi\Payment
 */
class Paypal implements PaymentInterface
{
    protected $clientId;
    protected $clientSecret;
    protected $paymentMode;

    /**
     * @var \Router
     */
    protected $router;

    /**
     * @param $config
     */
    public function __construct($config)
    {
        $this->clientId = $config['paypal']['client_id'];
        $this->clientSecret = $config['paypal']['client_secret'];
        $this->paymentMode = $config['paypal']['mode'];

        $this->router = Application::getRouter();
    }
    /**
     * @param Tpe $tpe
     * @param float $amount
     * @param array $params
     * @return mixed
     */
    public function request(Tpe $tpe, $amount, $params = array())
    {
        $amt = new Amount();
        $amt->setCurrency("EUR")
            ->setTotal($amount);

        $reference = $tpe->tpe_reference_id;

        $transaction = new Transaction();
        $transaction->setAmount($amt)
            ->setDescription("Transaction ".$reference)
            ->setInvoiceNumber($reference);

        $this->processTransaction($transaction);
    }

    /**
     * @param Panier $panier
     * @param $amount
     * @param $params
     * @return mixed
     */
    public function requestFromPanier(Panier $panier, $amount, $params = array())
    {
        $caddie = $panier->getProducts();

        $items = [];
        foreach ($caddie as $cad) {
            $item = (new Item())
                ->setName($cad['produit_nom_nom'])
                ->setCurrency('EUR')
                ->setQuantity($cad['quantite_order'])
                ->setPrice($cad['produit_total_ttc']);
            dump($cad);
            $items[] = $item;
        }

        $itemList = (new ItemList())->setItems($items);

        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();
        list($productsHT, $productsTTC) = $panier->getTotalProducts();
        $shipping = $panier->getShipping();

        $details = new Details();
        $details->setShipping($shipping->port_ttc)
            ->setTax($productsTTC - $productsHT)
            ->setSubtotal($productsHT);

        $amt = new Amount();
        $amt->setCurrency("EUR")
            ->setTotal($aPayerTTC)
            ->setDetails($details);

        $reference = $panier->getReference();

        $transaction = new Transaction();
        $transaction->setAmount($amt)
            ->setItemList($itemList)
            ->setDescription("Transaction ".$reference)
            ->setInvoiceNumber($reference);

        dump((object)$itemList);
        dump((object)$details);
        dump((object)$transaction);
        dump((object)$amt);

        $this->processTransaction($transaction);
    }

    /**
     * @param Transaction $transaction
     */
    protected function processTransaction(Transaction $transaction)
    {
        $payer = new Payer();
        $payer->setPaymentMethod("paypal");

        $redirectUrls = new RedirectUrls();
        $redirectUrls
            ->setReturnUrl(substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('traitement_actif', ['class' => 'paypal']))
            ->setCancelUrl(substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('order', ['_locale' => Boutique::getInstance()->getLangue()->ini]));


        $payment = new Payment();
        $payment->setIntent("sale")
            ->setPayer($payer)
            ->setRedirectUrls($redirectUrls)
            ->setTransactions(array($transaction));

        $approvalUrl = null;
        try {
            $payment->create($this->getApiContext());
            $approvalUrl = $payment->getApprovalLink();
        }catch(PayPalConnectionException $pce){
            // Don't spit out errors or use "exit" like this in production code
            echo '<pre>';print_r(json_decode($pce->getData()));
        }
die();
//        return $this->getFormTemplate($approvalUrl);

        if (empty($approvalUrl)){
            (new RedirectResponse($this->router->getRouter()->generate('order').'?error=paiementKO'))->send();
        }else {
            (new RedirectResponse($approvalUrl))->send();
        }
    }

    public function response($request){
        $payment = $this->executePayment([
            'paymentId' => $request['paymentId'],
            'PayerID' => $request['PayerID'],
        ]);

        $transactions = $payment->getTransactions();
        $reference = $transactions[0]->getInvoiceNumber();

        if ($payment->getState() == "approved") {
            try {
                $tpe = new Tpe;
                $orderId = $tpe->paiementOK($reference, $request);
            } catch (DediException $e) {
            }

            if (!empty($orderId)) {
                return new RedirectResponse($this->router->getRouter()->generate('commande', ['_locale' => Boutique::getInstance()->getLangue()->ini]));
            }
        }else{
            $tpe = new Tpe;
            $tpe->paiementKO($reference, $request);
        }
        return new RedirectResponse($this->router->getRouter()->generate('order', ['_locale' => Boutique::getInstance()->getLangue()->ini]).'?error=paiementKO');
    }
    /**
     * @param array $request
     * @return mixed
     */
    public function executePayment($request)
    {
        $apiContext = $this->getApiContext();
        // Get the payment Object by passing paymentId
        // payment id was previously stored in session in
        // CreatePaymentUsingPayPal.php
        $paymentId = $request['paymentId'];
        $payment = Payment::get($paymentId, $apiContext);

        // ### Payment Execute
        // PaymentExecution object includes information necessary
        // to execute a PayPal account payment.
        // The payer_id is added to the request query parameters
        // when the user is redirected from paypal back to your site
        $execution = new PaymentExecution();
        $execution->setPayerId($request['PayerID']);

        try {
            // Execute the payment
            $result = $payment->execute($execution, $apiContext);

            // ResultPrinter::printResult("Executed Payment", "Payment", $payment->getId(), $execution, $result);

            try {
                $payment = Payment::get($paymentId, $apiContext);
            } catch (\Exception $ex) {
                // ResultPrinter::printError("Get Payment", "Payment", null, null, $ex);
            }
        } catch (\Exception $ex) {
            // ResultPrinter::printError("Executed Payment", "Payment", null, null, $ex);
        }

        // ResultPrinter::printResult("Get Payment", "Payment", $payment->getId(), null, $payment);

        return $payment;
    }


    /**
     * @return ApiContext
     */
    protected function getApiContext()
    {
        // ### Api context
        // Use an ApiContext object to authenticate
        // API calls. The clientId and clientSecret for the
        // OAuthTokenCredential class can be retrieved from
        // developer.paypal.com

        $apiContext = new ApiContext(
            new OAuthTokenCredential(
                $this->clientId,
                $this->clientSecret
            )
        );

        // Comment this line out and uncomment the PP_CONFIG_PATH
        // 'define' block if you want to use static file
        // based configuration

        $apiContext->setConfig(
            array(
                'mode' => $this->paymentMode,
                'log.LogEnabled' => true,
                'log.FileName' => '../PayPal.log',
                'log.LogLevel' => 'DEBUG', // PLEASE USE `FINE` LEVEL FOR LOGGING IN LIVE ENVIRONMENTS
                'validation.level' => 'log',
                'cache.enabled' => true,
                // 'http.CURLOPT_CONNECTTIMEOUT' => 30
                // 'http.headers.PayPal-Partner-Attribution-Id' => '123123123'
            )
        );

        return $apiContext;
    }

    /**
     * @return string
     */
    public function getFormTemplate($url){
        $template = '<a href="'.$url.'">Paypal</a>';
        return $template;
    }
}